@@ -158,7 +158,11 @@ class CarnotPy:
     def get_token(self):
         result = requests.post(self.url + "/login/", json={"user_name": self.username, "password": self.password})
         assert result.status_code == 200
-        return result.json()["token"]
+        try:
+            token = result.json()["token"]
+            return token
+        except KeyError:
+            raise ValueError("Invalid username or password")
 
     def discover(self):
         token = self.get_token()
@@ -234,5 +238,31 @@ class CarnotPy:
         df.index = index
         return df
 
+    def get_sensor_id(self, facility_name, equipment_name, sensor_name):
+        token = self.get_token()
+        result = requests.post(self.url + "/point/",
+                               json={"facility_name": facility_name, "equipment_name": equipment_name},
+                               headers={"Authorization": "Bearer " + token, 'Content-Type': 'application/json'}
+                               )
+        assert result.status_code == 200
+        points = result.json()['result']
+        for point in points:
+            if point['sensor_name'] == sensor_name:
+                return point['sensor_id']
+        raise KeyError("Sensor not found")
 
+    def write(self, sensor_id, value, priority, duration=None):
+        token = self.get_token()
+        result = requests.put(
+            self.url + "/optimizationcontrol/",
+            json={
+                "sensor_id": sensor_id,
+                "value": value,
+                "priority": priority,
+                "duration": duration
+            },
+            headers={"Authorization": "Bearer " + token, 'Content-Type': 'application/json'}
+        )
+        assert result.status_code == 200
+        return result.json()
 
